import logging

from django.http import HttpRequest
from django.views import View
from rest_framework import permissions

from core.models import Reviewer, Student, Tutor

log = logging.getLogger(__name__)


class IsUserGenericPermission(permissions.BasePermission):
    """ Generic class that encapsulates how to identify someone
    as a member of a user Group """

    def has_permission(self, request: HttpRequest, view: View) -> bool:
        """ required by BasePermission. Check if user is instance of any
        of the models provided in class' models attribute """
        assert self.models is not None, (
            "'%s' has to include a `models` attribute"
            % self.__class__.__name__
        )

        user = request.user
        is_authorized = user.is_authenticated() and any(isinstance(
            user.get_associated_user(), models) for models in self.models)

        if not is_authorized:
            log.warn('User "%s" has no permission to view %s',
                     user.username, view.__class__.__name__)

        return is_authorized


class IsStudent(IsUserGenericPermission):
    """ Has student permissions """
    models = (Student,)


class IsReviewer(IsUserGenericPermission):
    """ Has reviewer permissions """
    models = (Reviewer,)


class IsTutor(IsUserGenericPermission):
    """ Has tutor permissions """
    models = (Tutor,)
