from django.test import TestCase

from core.models import (Feedback, Reviewer, Student, Submission,
                         SubmissionType, Tutor)
from util.factories import GradyUserFactory


class FeedbackTestCase(TestCase):

    factory = GradyUserFactory()

    def setUp(self):
        self.tutor = self.factory.make_tutor()
        self.student = self.factory.make_student()

        submission_type = SubmissionType.objects.create(
            name='Cooking some crystal with Jesse')
        Submission.objects.create(student=self.student, type=submission_type)
        Submission.assign_tutor(self.tutor)

    def test_can_assign_tutor(self):
        self.assertEqual(self.tutor.feedback_list.count(), 1)

    def test_feedback_origin_is_manual(self):
        feedback = self.tutor.feedback_list.all()[0]
        self.assertEqual(feedback.origin, Feedback.MANUAL)

    def test_feedback_status_is_editable(self):
        feedback = self.tutor.feedback_list.all()[0]
        self.assertEqual(feedback.status, Feedback.EDITABLE)


class FactoryTestCase(TestCase):

    factory = GradyUserFactory()

    def test_make_student(self):

        student = self.factory.make_student()

        self.assertEqual(Student.objects.count(), 1)
        self.assertEqual(student.exam, None)
        self.assertEqual(len(str(student.matrikel_no)), 8)

    def test_can_create_reviewer(self):
        self.assertTrue(isinstance(self.factory.make_reviewer(), Reviewer))

    def test_reviewer_appears_in_query_set(self):
        self.assertIn(self.factory.make_reviewer(), Reviewer.objects.all())

    def test_can_create_tutor(self):
        self.assertIn(self.factory.make_tutor(), Tutor.objects.all())

    def test_can_create_student(self):
        self.assertIn(self.factory.make_student(), Student.objects.all())


class AccountsTestCase(TestCase):

    factory = GradyUserFactory()

    def _test_user_obj_returns_correct_type(self, maker):
        model = maker()
        user = model.user
        self.assertEqual(user.get_associated_user(), model)

    def test_user_obj_returns_correct_type_student(self):
        self._test_user_obj_returns_correct_type(self.factory.make_student)

    def test_user_obj_returns_correct_type_tutor(self):
        self._test_user_obj_returns_correct_type(self.factory.make_tutor)

    def test_user_obj_returns_correct_type_reviewer(self):
        self._test_user_obj_returns_correct_type(self.factory.make_reviewer)
