from django.conf.urls import include, url
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.views.generic.base import TemplateView
from rest_framework.routers import DefaultRouter
from rest_framework_jwt.views import obtain_jwt_token, refresh_jwt_token

from core import views

# Create a router and register our viewsets with it.
router = DefaultRouter()
router.register(r'student', views.StudentReviewerApiViewSet)
router.register(r'examtype', views.ExamApiViewSet)
router.register(r'submissiontype', views.SubmissionTypeApiView)
router.register(r'tutor', views.TutorApiViewSet)

# regular views that are not viewsets
regular_views_urlpatterns = [
    url(r'student-page', views.StudentSelfApiView.as_view(), name='student-page'),
    url(r'user-role', views.get_user_role, name='user-role'),
    url(r'exam-module', views.get_exam_module, name='exam-module'),
    url(r'jwt-time-delta', views.get_jwt_expiration_delta, name='jwt-time-delta')
]

urlpatterns = [
    url(r'^api/', include(router.urls)),
    url(r'^api/', include(regular_views_urlpatterns)),
    url(r'^api-token-auth/', obtain_jwt_token),
    url(r'^api-token-refresh', refresh_jwt_token),
    url(r'^$', TemplateView.as_view(template_name='index.html')),
]

urlpatterns += staticfiles_urlpatterns()
